from PySide6.QtCore import QThread
from PySide6.QtCore import Signal
from PySide6.QtWidgets import QComboBox
from PySide6.QtWidgets import QLabel
from PySide6.QtWidgets import QWidget
from PySide6.QtWidgets import QPushButton

from windows.post_processing.post_processing_page import Ui_PostProcessing
from windows.post_processing.post_processing_column import Ui_LeftColumn

# toolkit PySide6 Widgets
from ansys.aedt.toolkits.common.ui.utils.widgets import PyLabel


class ValidateAnalyzeThread(QThread):
    finished_signal = Signal(bool)

    def __init__(self, app, selected_project, selected_design):
        super().__init__()
        self.post_processing_menu = app
        self.main_window = app.main_window
        self.selected_project = selected_project
        self.selected_design = selected_design
        self.is_finished = False

    def run(self):
        self.is_finished = self.main_window.val_check_and_analysis(
            self.selected_project, self.selected_design
        )
        self.finished_signal.emit(self.is_finished)


class MagnetLossThread(QThread):
    finished_signal = Signal(bool)

    def __init__(self, app, selected_project, selected_design):
        super().__init__()
        self.post_processing_menu = app
        self.main_window = app.main_window
        self.selected_project = selected_project
        self.selected_design = selected_design
        self.is_finished = False

    def run(self):
        self.is_finished = self.main_window.get_report(
            self.selected_project, self.selected_design
        )
        self.finished_signal.emit(self.is_finished)


class PostProcessingMenu(object):
    def __init__(self, main_window):
        # General properties
        self.main_window = main_window
        self.ui = main_window.ui

        # Add page
        postprocessing_menu_index = self.ui.add_page(Ui_PostProcessing)
        self.ui.load_pages.pages.setCurrentIndex(postprocessing_menu_index)
        self.postprocessing_menu_widget = self.ui.load_pages.pages.currentWidget()

        # Add left column
        new_column_widget = QWidget()
        new_ui = Ui_LeftColumn()
        new_ui.setupUi(new_column_widget)
        self.ui.left_column.menus.menus.addWidget(new_column_widget)
        self.postprocessing_column_widget = new_column_widget
        self.postprocessing_column_vertical_layout = new_ui.postprocessing_vertical_layout

        # Specific properties
        # Labels
        self.setup_name_label = self.postprocessing_menu_widget.findChild(QLabel, "setup_name_label")
        # Combobox
        self.setup_name_combo = self.postprocessing_menu_widget.findChild(QComboBox, "setup_name_combo")
        # Buttons
        self.validate_and_analyze_button = self.postprocessing_menu_widget.findChild(QPushButton,
                                                                                     "validate_and_analyze")
        self.get_magnet_loss_button = self.postprocessing_menu_widget.findChild(QPushButton, "get_magnet_loss")
        self.validate_analyze_thread = None
        self.magnet_loss_thread = None

    def setup(self):
        # Modify theme
        app_color = self.main_window.ui.themes["app_color"]
        text_color = app_color["text_active"]
        background = app_color["dark_three"]

        self.validate_and_analyze_button.clicked.connect(self.validate_analyze_button_clicked)
        self.get_magnet_loss_button.clicked.connect(self.magnet_loss_button_clicked)

        # UI from Designer
        # Multiplier label button
        multiplier_label_style = """
                            QLabel {{
                            color: {_color};
                            font-size: {_font_size}pt;
                            font-weight: bold;
                            }}
                            """
        custom_style = multiplier_label_style.format(
            _color=text_color, _bg_color=background, _font_size=self.main_window.properties.font["title_size"]
        )
        self.setup_name_label.setStyleSheet(custom_style)

        # Multiplier label button
        multiplier_label_style = """
                    QLabel {{
                    color: {_color};
                    font-size: {_font_size}pt;
                    font-weight: bold;
                    }}
                    """
        custom_style = multiplier_label_style.format(
            _color=text_color, _bg_color=background, _font_size=self.main_window.properties.font["title_size"]
        )
        self.validate_and_analyze_button.setStyleSheet(custom_style)
        self.get_magnet_loss_button.setStyleSheet(custom_style)

        # Combo boxes
        combo_box_style = """
                    QComboBox {{
                        border: none;
                        padding: 10px;
                        color: {_color};
                        background-color: {_bg_color};
                        selection-background-color: red;
                        font-size: {_font_size}pt;
                    }}
                    QComboBox QAbstractItemView {{
                        border: none;
                        background-color: {_bg_color};
                        color: {_color};
                    }}
                """
        custom_style = combo_box_style.format(
            _color=text_color, _bg_color=background, _font_size=self.main_window.properties.font["title_size"]
        )
        self.setup_name_combo.setStyleSheet(custom_style)

        # Set Column
        msg = "Post-Processing"
        label_widget = PyLabel(
            text=msg,
            font_size=self.ui.app.properties.font["title_size"],
            color=self.ui.themes["app_color"]["text_description"])
        self.postprocessing_column_vertical_layout.addWidget(label_widget)

    def validate_analyze_button_clicked(self):
        if not self.main_window.check_connection():
            msg = "Backend not running."
            self.ui.update_logger(msg)
            return False

        if (self.validate_analyze_thread and self.validate_analyze_thread.isRunning()
                or self.magnet_loss_thread and self.magnet_loss_thread.isRunning()
                or self.main_window.backend_busy()):
            msg = "Toolkit running"
            self.ui.update_logger(msg)
            self.main_window.logger.debug(msg)
            return False

        be_properties = self.main_window.get_properties()

        be_properties["setup_to_analyze"] = self.setup_name_combo.currentText()

        self.main_window.set_properties(be_properties)

        if be_properties.get("active_project"):
            self.ui.update_progress(0)
            selected_project = self.main_window.home_menu.project_combobox.currentText()
            selected_design = self.main_window.home_menu.design_combobox.currentText()

            # Start a separate thread for the backend call
            self.validate_analyze_thread = ValidateAnalyzeThread(
                app=self,
                selected_project=selected_project,
                selected_design=selected_design,
            )
            self.validate_analyze_thread.finished_signal.connect(self.validate_analyze_process_finished)

            self.validate_analyze_thread.start()
        else:
            self.ui.update_logger("Toolkit not connected to AEDT.")

    def magnet_loss_button_clicked(self):
        if not self.main_window.check_connection():
            msg = "Backend not running."
            self.ui.update_logger(msg)
            return False

        if (self.validate_analyze_thread and self.validate_analyze_thread.isRunning()
                or self.magnet_loss_thread and self.magnet_loss_thread.isRunning()
                or self.main_window.backend_busy()):
            msg = "Toolkit running"
            self.ui.update_logger(msg)
            self.main_window.logger.debug(msg)
            return False

        be_properties = self.main_window.get_properties()

        if be_properties.get("active_project"):
            self.ui.update_progress(0)
            selected_project = self.main_window.home_menu.project_combobox.currentText()
            selected_design = self.main_window.home_menu.design_combobox.currentText()

            # Start a separate thread for the backend call
            self.magnet_loss_thread = MagnetLossThread(
                app=self,
                selected_project=selected_project,
                selected_design=selected_design,
            )
            self.magnet_loss_thread.finished_signal.connect(self.magnet_loss_process_finished)

            self.magnet_loss_thread.start()
        else:
            self.ui.update_logger("Toolkit not connected to AEDT.")

    def validate_analyze_process_finished(self):
        self.ui.update_progress(100)
        properties = self.main_window.get_properties()
        active_design = properties["active_design"]

        self.main_window.home_menu.update_design()
        self.main_window.home_menu.design_combobox.setCurrentText(active_design)

        if self.validate_analyze_thread.is_finished:
            msg = "Validation and analysis successful."
            self.ui.update_logger(msg)
        else:
            msg = f"Failed backend call: {self.main_window.url}"
            self.ui.update_logger(msg)

    def magnet_loss_process_finished(self):
        self.ui.update_progress(100)
        properties = self.main_window.get_properties()
        active_design = properties["active_design"]

        self.main_window.home_menu.update_design()
        self.main_window.home_menu.design_combobox.setCurrentText(active_design)

        if self.magnet_loss_thread.is_finished:
            msg = "Magnet loss computation successful."
            self.ui.update_logger(msg)
        else:
            msg = f"Failed backend call: {self.main_window.url}"
            self.ui.update_logger(msg)

    def get_setups(self):
        backend_properties = self.ui.app.get_properties()
        setups = []
        if backend_properties.get("active_project") and backend_properties.get("active_design"):
            setups = self.main_window.get_design_setups()
        self.setup_name_combo.clear()
        if setups:
            for setup in setups:
                self.setup_name_combo.addItem(setup)



from PySide6 import QtCore, QtGui, QtWidgets
from PySide6.QtWidgets import QLabel
from PySide6.QtWidgets import QGridLayout
from PySide6.QtWidgets import QWidget

from windows.help.help_column import Ui_LeftColumn
from windows.help.help_page import Ui_help

import tempfile
from ansys.aedt.toolkits.magnet_segmentation import __version__

DOCUMENTATION_URL = "https://magnet.segmentation.toolkit.docs.pyansys.com/"
ISSUE_TRACKER_URL = "https://github.com/ansys/magnet-segmentation-toolkit/issues"
ABOUT_TEXT = f"""<h2>Magnet Segmentation Toolkit {__version__}</h2>
<p>Project using <a href='https://wiki.qt.io/Qt_for_Python'> PySide6</a>, Copyright 2024 The Qt Company Ltd.</p>
<p>The graphical user interface (GUI) components are licensed under <a href='https://www.gnu.org/licenses/lgpl-3.0.en.html'>LGPL v3.0</a>.</p> 
<p>Except for the GUI components, your use of this software is governed by the MIT License. In addition, this package allows you to access a software that is licensed under separate terms ("Separately Licensed Software"). If you choose to install such Separately Licensed Software, you acknowledge that you are responsible for complying with any associated terms and conditions.</p>
<p>Copyright 2023 - 2024 ANSYS, Inc. All rights reserved.</p>
<p>If you have any questions or issues, please open an issue in <a href='{ISSUE_TRACKER_URL}'>pyaedt-toolkits-antenna Issues</a> page.</p>
<p>Alternatively, you can contact us at <a href='mailto:pyansys.core@ansys.com'>pyansys.core@ansys.com</a>.</p>
"""


class HelpMenu(object):
    def __init__(self, main_window):
        # General properties
        self.main_window = main_window
        self.ui = main_window.ui
        self.temp_folder = tempfile.mkdtemp()

        # Add page
        help_menu_index = self.ui.add_page(Ui_help)
        self.ui.load_pages.pages.setCurrentIndex(help_menu_index)
        self.help_menu_widget = self.ui.load_pages.pages.currentWidget()

        # Add left column
        new_column_widget = QWidget()
        new_ui = Ui_LeftColumn()
        new_ui.setupUi(new_column_widget)
        self.ui.left_column.menus.menus.addWidget(new_column_widget)
        self.help_column_widget = new_column_widget
        self.help_column_vertical_layout = new_ui.help_vertical_layout

        # Specific properties
        self.help_label = self.help_menu_widget.findChild(QLabel, "help_label")
        self.help_grid = self.help_menu_widget.findChild(QGridLayout, "help_grid")

        self.help_button_layout = None
        self.help_button = None
        self.online_documentation_button = None
        self.issue_tracker_button = None

    def setup(self):
        # Modify theme
        app_color = self.main_window.ui.themes["app_color"]
        text_color = app_color["text_active"]
        background = app_color["dark_three"]

        # Label button
        help_label_style = """
        QLabel {{
        color: {_color};
        font-size: {_font_size}pt;
        font-weight: bold;
        }}
        """
        custom_style = help_label_style.format(
            _color=text_color, _bg_color=background, _font_size=self.main_window.properties.font["title_size"]
        )
        self.help_label.setStyleSheet(custom_style)

        # Set column

        # About button
        row_returns = self.ui.add_n_buttons(
            self.help_column_vertical_layout, num_buttons=1,
            height=40,
            width=[200],
            text=["About"],
            font_size=self.main_window.properties.font["title_size"]
        )
        self.help_button_layout = row_returns[0]
        self.help_button = row_returns[1]
        self.help_button_layout.addWidget(self.help_button)
        self.help_button.clicked.connect(self.about_button_clicked)

        # Documentation button
        row_returns = self.ui.add_n_buttons(
            self.help_column_vertical_layout, num_buttons=1,
            height=40,
            width=[200],
            text=["Documentation website"],
            font_size=self.main_window.properties.font["title_size"]
        )
        self.help_button_layout = row_returns[0]
        self.online_documentation_button = row_returns[1]
        self.help_button_layout.addWidget(self.online_documentation_button)
        self.online_documentation_button.clicked.connect(self.visit_website)

        # Issue tracker button
        row_returns = self.ui.add_n_buttons(
            self.help_column_vertical_layout, num_buttons=1,
            height=40,
            width=[200],
            text=["Issue tracker"],
            font_size=self.main_window.properties.font["title_size"]
        )

        self.help_button_layout = row_returns[0]
        self.issue_tracker_button = row_returns[1]
        self.help_button_layout.addWidget(self.issue_tracker_button)
        self.issue_tracker_button.clicked.connect(self.report_issue)

    def about_button_clicked(self):
        """Display the PyAEDT Magnet Segmentation Toolkit 'About' information."""

        mbox = QtWidgets.QMessageBox.about(self.main_window, "About", ABOUT_TEXT)

    def visit_website(self):
        """Access the PyAEDT Magnet Segmentation Toolkit documentation."""
        url = QtCore.QUrl(DOCUMENTATION_URL)
        QtGui.QDesktopServices.openUrl(url)

    def report_issue(self):
        """Access the PyAEDT Magnet Segmentation Toolkit issues tracker."""
        url = QtCore.QUrl(ISSUE_TRACKER_URL)
        QtGui.QDesktopServices.openUrl(url)



# Copyright (C) 2023 - 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Data classes used to store data related to various settings.

The settings include common AEDT toolkit settings and settings associated
to motor workflows in AEDT.
"""
import os
import sys

from ansys.aedt.toolkits.common.backend.models import CommonProperties
from ansys.aedt.toolkits.common.backend.models import common_properties
from pydantic import BaseModel

if sys.version_info >= (3, 11):
    import tomllib
else:
    import tomli as tomllib


class AEDTProperties(BaseModel):
    """Store AEDT properties."""

    is_skewed: bool = False
    apply_mesh_sheets: bool = False
    magnets_material: str = ""
    rotor_material: str = ""
    stator_material: str = ""
    rotor_slices: int = 0
    magnet_segments_per_slice: int = 0
    mesh_sheets_number: int = 0
    skew_angle: str = ""
    setup_to_analyze: str = "Setup1"
    objects: list = []


class Properties(CommonProperties, AEDTProperties, validate_assignment=True):
    """Store all properties."""


backend_properties = {}
if os.path.expanduser(os.path.join(os.path.dirname(__file__), "aedt_properties.toml")):
    with open(os.path.join(os.path.dirname(__file__), "aedt_properties.toml"), mode="rb") as file_handler:
        backend_properties = tomllib.load(file_handler)

toolkit_property = {}
if backend_properties:
    for backend_key in backend_properties:
        if hasattr(common_properties, backend_key):
            setattr(common_properties, backend_key, backend_properties[backend_key])
        else:
            toolkit_property[backend_key] = backend_properties[backend_key]

new_common_properties = {}
for common_key in common_properties:
    new_common_properties[common_key[0]] = common_key[1]

properties = Properties(**toolkit_property, **new_common_properties)



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'post_processing_page.ui'
##
## Created by: Qt User Interface Compiler version 6.6.1
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QComboBox, QHBoxLayout, QLabel,
    QPushButton, QSizePolicy, QSpacerItem, QVBoxLayout,
    QWidget)

class Ui_PostProcessing(object):
    def setupUi(self, PostProcessing):
        if not PostProcessing.objectName():
            PostProcessing.setObjectName(u"PostProcessing")
        PostProcessing.resize(890, 1178)
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(PostProcessing.sizePolicy().hasHeightForWidth())
        PostProcessing.setSizePolicy(sizePolicy)
        PostProcessing.setProperty("post-processing", QRect(9, 9, 701, 441))
        self.verticalLayout = QVBoxLayout(PostProcessing)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.verticalLayout_3 = QVBoxLayout()
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.setup_name_label = QLabel(PostProcessing)
        self.setup_name_label.setObjectName(u"setup_name_label")
        sizePolicy1 = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.setup_name_label.sizePolicy().hasHeightForWidth())
        self.setup_name_label.setSizePolicy(sizePolicy1)
        self.setup_name_label.setMinimumSize(QSize(0, 0))

        self.horizontalLayout_2.addWidget(self.setup_name_label)

        self.horizontalSpacer_6 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout_2.addItem(self.horizontalSpacer_6)

        self.setup_name_combo = QComboBox(PostProcessing)
        self.setup_name_combo.addItem("")
        self.setup_name_combo.setObjectName(u"setup_name_combo")
        sizePolicy2 = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Fixed)
        sizePolicy2.setHorizontalStretch(0)
        sizePolicy2.setVerticalStretch(0)
        sizePolicy2.setHeightForWidth(self.setup_name_combo.sizePolicy().hasHeightForWidth())
        self.setup_name_combo.setSizePolicy(sizePolicy2)
        self.setup_name_combo.setMinimumSize(QSize(300, 0))

        self.horizontalLayout_2.addWidget(self.setup_name_combo)


        self.verticalLayout_3.addLayout(self.horizontalLayout_2)

        self.verticalSpacer = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Maximum)

        self.verticalLayout_3.addItem(self.verticalSpacer)

        self.validate_and_analyze = QPushButton(PostProcessing)
        self.validate_and_analyze.setObjectName(u"validate_and_analyze")
        sizePolicy3 = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy3.setHorizontalStretch(0)
        sizePolicy3.setVerticalStretch(0)
        sizePolicy3.setHeightForWidth(self.validate_and_analyze.sizePolicy().hasHeightForWidth())
        self.validate_and_analyze.setSizePolicy(sizePolicy3)
        self.validate_and_analyze.setMinimumSize(QSize(0, 40))
        self.validate_and_analyze.setMaximumSize(QSize(16777215, 16777215))

        self.verticalLayout_3.addWidget(self.validate_and_analyze)

        self.get_magnet_loss = QPushButton(PostProcessing)
        self.get_magnet_loss.setObjectName(u"get_magnet_loss")
        sizePolicy3.setHeightForWidth(self.get_magnet_loss.sizePolicy().hasHeightForWidth())
        self.get_magnet_loss.setSizePolicy(sizePolicy3)
        self.get_magnet_loss.setMinimumSize(QSize(0, 40))
        self.get_magnet_loss.setMaximumSize(QSize(16777215, 16777215))

        self.verticalLayout_3.addWidget(self.get_magnet_loss)


        self.horizontalLayout.addLayout(self.verticalLayout_3)

        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Preferred, QSizePolicy.Minimum)

        self.horizontalLayout.addItem(self.horizontalSpacer)


        self.verticalLayout.addLayout(self.horizontalLayout)


        self.retranslateUi(PostProcessing)

        QMetaObject.connectSlotsByName(PostProcessing)
    # setupUi

    def retranslateUi(self, PostProcessing):
        PostProcessing.setWindowTitle(QCoreApplication.translate("PostProcessing", u"Form", None))
        self.setup_name_label.setText(QCoreApplication.translate("PostProcessing", u"Setup to analyze", None))
        self.setup_name_combo.setItemText(0, QCoreApplication.translate("PostProcessing", u"--No Setups--", None))

        self.validate_and_analyze.setText(QCoreApplication.translate("PostProcessing", u"Validate and Analyze", None))
        self.get_magnet_loss.setText(QCoreApplication.translate("PostProcessing", u"Get Magnet Loss", None))
    # retranslateUi




# Copyright (C) 2023 - 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Data classes used to store data related to UI settings."""
import os
import sys

from ansys.aedt.toolkits.common.ui.models import UIProperties
from ansys.aedt.toolkits.common.ui.models import general_settings

if sys.version_info >= (3, 11):
    import tomllib
else:
    import tomli as tomllib


class Properties(UIProperties, validate_assignment=True):
    """Store all properties."""


frontend_properties = {}
if os.path.exists(os.path.join(os.path.dirname(__file__), "frontend_properties.toml")):
    with open(os.path.join(os.path.dirname(__file__), "frontend_properties.toml"), mode="rb") as file_handler:
        frontend_properties = tomllib.load(file_handler)

toolkit_property = {}
if frontend_properties:
    for frontend_key in frontend_properties:
        if frontend_key == "defaults":
            for toolkit_key in frontend_properties["defaults"]:
                if hasattr(general_settings, toolkit_key):
                    setattr(general_settings, toolkit_key, frontend_properties["defaults"][toolkit_key])
        else:
            toolkit_property[frontend_key] = frontend_properties[frontend_key]

new_common_properties = {}
for common_key in general_settings:
    new_common_properties[common_key[0]] = common_key[1]

properties = Properties(**toolkit_property, **new_common_properties)



# Copyright (C) 2023 - 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import os
import sys

# PySide6 Widgets
from PySide6.QtWidgets import QApplication
from PySide6.QtWidgets import QMainWindow

# ToolkitBackend frontend API
from actions import Frontend
from ansys.aedt.toolkits.common.ui.common_windows.home_menu import HomeMenu
from ansys.aedt.toolkits.common.ui.common_windows.settings_column import SettingsMenu

# Import general common frontend modules
from ansys.aedt.toolkits.common.ui.logger_handler import logger

# Common windows
from ansys.aedt.toolkits.common.ui.main_window.main_window_layout import MainWindowLayout

# Default user interface properties
from models import properties
from windows.help.help_menu import HelpMenu
from windows.plot_design.plot_design_menu import PlotDesignMenu
from windows.post_processing.post_processing_menu import PostProcessingMenu

# New windows
from windows.segmentation.segmentation_menu import SegmentationMenu

# Windows


# Backend URL and port
url = properties.backend_url
port = properties.backend_port

os.environ["QT_API"] = "pyside6"
os.environ["QT_FONT_DPI"] = "96"

if properties.high_resolution:
    os.environ["QT_SCALE_FACTOR"] = "2"


class ApplicationWindow(QMainWindow, Frontend):
    def __init__(self):
        super().__init__()

        self.thread = None
        self.properties = properties

        # General Settings

        # Create main window layout
        self.ui = MainWindowLayout(self)
        self.ui.setup()

        # Settings menu
        self.settings_menu = SettingsMenu(self)
        self.settings_menu.setup()
        self.ui.title_bar.clicked.connect(self.settings_menu_clicked)

        # Check backend connection
        success = self.check_connection()

        if not success:
            msg = "Error getting properties from backend. User interface running without backend"
            self.ui.update_logger(msg)
            logger.error(msg)
            self.settings_menu.signal_flag = False
            self.settings_menu.aedt_version.addItem("Backend OFF")
            self.settings_menu.aedt_session.addItem("Backend OFF")
        else:
            # Get default properties
            be_properties = self.get_properties()
            # Get AEDT installed versions
            installed_versions = self.installed_versions()

            self.settings_menu.aedt_session.clear()
            self.settings_menu.aedt_session.addItem("New Session")
            if installed_versions:
                self.settings_menu.connect_aedt.setEnabled(True)
                for ver in installed_versions:
                    self.settings_menu.aedt_version.addItem(ver)
            else:
                self.settings_menu.aedt_version.addItem("AEDT not installed")

            if be_properties.get("aedt_version") in installed_versions:
                self.settings_menu.aedt_version.setCurrentText(be_properties.get("aedt_version"))

        # Custom toolkit setup starts here

        # Home menu
        self.home_menu = HomeMenu(self)
        self.home_menu.setup()
        self.ui.left_menu.clicked.connect(self.home_menu_clicked)

        # Segmentation menu
        self.segmentation_menu = SegmentationMenu(self)
        self.segmentation_menu.setup()
        self.ui.left_menu.clicked.connect(self.segmentation_menu_clicked)

        # Post-processing menu
        self.post_processing_menu = PostProcessingMenu(self)
        self.post_processing_menu.setup()
        self.ui.left_menu.clicked.connect(self.post_processing_menu_clicked)

        # Plot design menu
        self.plot_design_menu = PlotDesignMenu(self)
        self.plot_design_menu.setup()
        self.ui.left_menu.clicked.connect(self.plot_design_menu_clicked)

        # Help menu
        self.help_menu = HelpMenu(self)
        self.help_menu.setup()
        self.ui.left_menu.clicked.connect(self.help_menu_clicked)

        # Close column
        self.ui.title_bar.clicked.connect(self.close_menu_clicked)
        self.ui.left_menu.clicked.connect(self.progress_menu_clicked)
        self.ui.left_column.clicked.connect(self.close_menu_clicked)

        # Home page as first page
        self.ui.set_page(self.ui.load_pages.home_page)

    def home_menu_clicked(self):
        selected_menu = self.ui.get_selected_menu()
        menu_name = selected_menu.objectName()
        self.ui.left_menu.select_only_one(selected_menu.objectName())
        if menu_name == "home_menu":
            selected_menu.set_active(True)
            self.ui.set_page(self.ui.load_pages.home_page)

            is_left_visible = self.ui.is_left_column_visible()

            self.ui.set_left_column_menu(
                menu=self.ui.left_column.menus.menu_home,
                title="Home",
                icon_path=self.ui.images_load.icon_path("icon_home.svg"),
            )

            if not is_left_visible:
                self.ui.toggle_left_column()

    def settings_menu_clicked(self):
        selected_menu = self.ui.get_selected_menu()
        menu_name = selected_menu.objectName()
        is_right_visible = self.ui.is_right_column_visible()
        is_left_visible = self.ui.is_left_column_visible()

        if menu_name == "top_settings" and not is_right_visible:
            self.ui.app.settings_menu.show_widgets()
            if is_left_visible:
                self.ui.toggle_left_column()
                self.ui.left_menu.deselect_all()
            self.ui.toggle_right_column()
            self.ui.set_right_column_menu(title="Settings")

    def close_menu_clicked(self):
        selected_menu = self.ui.get_selected_menu()
        menu_name = selected_menu.objectName()
        if menu_name != "top_settings" and self.ui.is_left_column_visible():
            selected_menu.set_active(False)
            self.ui.toggle_left_column()
            self.ui.left_menu.deselect_all()
        if menu_name == "top_settings" and self.ui.is_right_column_visible():
            self.ui.toggle_right_column()

    def progress_menu_clicked(self):
        selected_menu = self.ui.get_selected_menu()
        menu_name = selected_menu.objectName()
        self.ui.left_menu.select_only_one(selected_menu.objectName())
        if menu_name == "progress_menu":
            is_progress_visible = self.ui.is_progress_visible()
            if is_progress_visible:
                selected_menu.set_active(False)
            self.ui.toggle_progress()

    def segmentation_menu_clicked(self):
        selected_menu = self.ui.get_selected_menu()
        menu_name = selected_menu.objectName()

        if menu_name == "segmentation_menu":
            selected_menu.set_active(True)
            self.ui.set_page(self.segmentation_menu.segmentation_menu_widget)

            is_left_visible = self.ui.is_left_column_visible()

            self.ui.set_left_column_menu(
                menu=self.segmentation_menu.segmentation_column_widget,
                title="Segmentation",
                icon_path=self.ui.images_load.icon_path("motor.svg"),
            )

            if not is_left_visible:
                self.ui.toggle_left_column()

            # Check backend connection
            success = self.check_connection()

            if success:
                # msg = "Loading materials..."
                # self.ui.update_logger(msg)
                # logger.info(msg)
                self.segmentation_menu.get_materials()

    def post_processing_menu_clicked(self):
        selected_menu = self.ui.get_selected_menu()
        menu_name = selected_menu.objectName()

        if menu_name == "post_processing_menu":
            selected_menu.set_active(True)
            self.ui.set_page(self.post_processing_menu.postprocessing_menu_widget)

            is_left_visible = self.ui.is_left_column_visible()

            self.ui.set_left_column_menu(
                menu=self.post_processing_menu.postprocessing_column_widget,
                title="PostProcessing",
                icon_path=self.ui.images_load.icon_path("icon_plot_2d.svg"),
            )

            if not is_left_visible:
                self.ui.toggle_left_column()

            # Check backend connection
            success = self.check_connection()

            if success:
                # msg = "Loading setups..."
                # self.ui.update_logger(msg)
                # logger.info(msg)
                self.post_processing_menu.get_setups()

    def plot_design_menu_clicked(self):
        selected_menu = self.ui.get_selected_menu()
        menu_name = selected_menu.objectName()

        if menu_name == "plot_design_menu":
            selected_menu.set_active(True)
            self.ui.set_page(self.plot_design_menu.plot_design_menu_widget)

            self.ui.set_left_column_menu(
                menu=self.plot_design_menu.plot_design_column_widget,
                title="Plot Design",
                icon_path=self.ui.images_load.icon_path("cylindrical_data_graphic.svg"),
            )

            is_left_visible = self.ui.is_left_column_visible()
            if not is_left_visible:
                self.ui.toggle_left_column()

    def help_menu_clicked(self):
        selected_menu = self.ui.get_selected_menu()
        menu_name = selected_menu.objectName()

        if menu_name == "help_menu":
            selected_menu.set_active(True)
            self.ui.set_page(self.help_menu.help_menu_widget)

            # TODO: Update path once help menu is released
            self.ui.set_left_column_menu(
                menu=self.help_menu.help_column_widget,
                title="Help",
                icon_path=self.ui.images_load.icon_path("help.svg"),
            )

            is_left_visible = self.ui.is_left_column_visible()
            if not is_left_visible:
                self.ui.toggle_left_column()


if __name__ == "__main__":
    app = QApplication(sys.argv)
    window = ApplicationWindow()
    window.show()
    sys.exit(app.exec())



from PySide6.QtCore import QThread
from PySide6.QtCore import Signal
from PySide6.QtWidgets import QLabel
from PySide6.QtWidgets import QGridLayout
from PySide6.QtWidgets import QWidget
from windows.plot_design.plot_design_column import Ui_LeftColumn
from windows.plot_design.plot_design_page import Ui_Plot_Design

import tempfile
from pyvistaqt import BackgroundPlotter
import base64
import pyvista as pv
import os


class PlotDesignThread(QThread):
    finished_signal = Signal(bool)

    def __init__(self, app, selected_project, selected_design):
        super().__init__()
        self.plot_design_menu = app
        self.main_window = app.main_window
        self.selected_project = selected_project
        self.selected_design = selected_design
        self.model_info = None

    def run(self):
        be_properties = self.main_window.get_properties()
        self.model_info = self.main_window.get_aedt_model(
            self.selected_project, self.selected_design, air_objects=False, obj_list=be_properties["objects"]
        )
        self.finished_signal.emit(self.model_info)


class PlotDesignMenu(object):
    def __init__(self, main_window):
        # General properties
        self.main_window = main_window
        self.ui = main_window.ui
        self.temp_folder = tempfile.mkdtemp()

        # Add page
        plot_design_menu_index = self.ui.add_page(Ui_Plot_Design)
        self.ui.load_pages.pages.setCurrentIndex(plot_design_menu_index)
        self.plot_design_menu_widget = self.ui.load_pages.pages.currentWidget()

        # Add left column
        new_column_widget = QWidget()
        new_ui = Ui_LeftColumn()
        new_ui.setupUi(new_column_widget)
        self.ui.left_column.menus.menus.addWidget(new_column_widget)
        self.plot_design_column_widget = new_column_widget
        self.plot_design_column_vertical_layout = new_ui.plot_design_vertical_layout

        # Specific properties
        self.plot_design_label = self.plot_design_menu_widget.findChild(QLabel, "plot_design_label")
        self.plot_design_grid = self.plot_design_menu_widget.findChild(QGridLayout, "plot_design_grid")

        self.plot_design_button_layout = None
        self.plot_design_button = None
        self.get_model_thread = None

    def setup(self):
        # Modify theme
        app_color = self.main_window.ui.themes["app_color"]
        text_color = app_color["text_active"]
        background = app_color["dark_three"]

        # Geometry label button
        plot_design_label_style = """
                                    QLabel {{
                                    color: {_color};
                                    font-size: {_font_size}pt;
                                    font-weight: bold;
                                    }}
                                    """
        custom_style = plot_design_label_style.format(
            _color=text_color, _bg_color=background, _font_size=self.main_window.properties.font["title_size"]
        )
        self.plot_design_label.setStyleSheet(custom_style)

        # Set column
        # Create geometry button
        row_returns = self.ui.add_n_buttons(
            self.plot_design_column_vertical_layout, num_buttons=1,
            height=40,
            width=[200],
            text=["View Model"],
            font_size=self.main_window.properties.font["title_size"]
        )

        self.plot_design_button_layout = row_returns[0]
        self.plot_design_button = row_returns[1]
        self.plot_design_button_layout.addWidget(self.plot_design_button)
        self.plot_design_button.clicked.connect(self.plot_design_button_clicked)

    def plot_design_button_clicked(self):

        if not self.main_window.check_connection():
            msg = "Backend not running."
            self.ui.update_logger(msg)
            return False

        if self.get_model_thread and self.get_model_thread.isRunning() or self.main_window.backend_busy():
            msg = "ToolkitBackend running"
            self.ui.update_logger(msg)
            self.main_window.logger.debug(msg)
            return False

        be_properties = self.main_window.get_properties()

        if be_properties.get("active_project"):
            self.ui.update_progress(0)
            selected_project = self.main_window.home_menu.project_combobox.currentText()
            selected_design = self.main_window.home_menu.design_combobox.currentText()
            # Start a separate thread for the backend call
            self.get_model_thread = PlotDesignThread(
                app=self,
                selected_project=selected_project,
                selected_design=selected_design,
            )
            self.get_model_thread.finished_signal.connect(self.get_model_finished)

            msg = "Exporting model."
            self.ui.update_logger(msg)

            self.get_model_thread.start()

        else:
            self.ui.update_logger("ToolkitBackend not connect to AEDT.")

    def get_model_finished(self):
        self.ui.update_progress(100)

        self.main_window.ui.clear_layout(self.plot_design_grid)

        if self.get_model_thread.model_info:
            model_info = self.get_model_thread.model_info

            plotter = BackgroundPlotter(show=False)

            for element in model_info:
                # Decode response
                encoded_data = model_info[element][0]
                encoded_data_bytes = bytes(encoded_data, "utf-8")
                decoded_data = base64.b64decode(encoded_data_bytes)
                # Create obj file locally
                file_path = os.path.join(self.temp_folder, element + ".obj")
                with open(file_path, "wb") as f:
                    f.write(decoded_data)
                # Create PyVista object
                if not os.path.exists(file_path):
                    break

                cad_mesh = pv.read(file_path)

                plotter.add_mesh(
                    cad_mesh, color=model_info[element][1], show_scalar_bar=False, opacity=model_info[element][2]
                )

            plotter.view_isometric()
            plotter.set_background(color=self.main_window.ui.themes["app_color"]["bg_one"])
            plotter.add_axes_at_origin(labels_off=True, line_width=5)
            plotter.show_grid(color=self.main_window.ui.themes["app_color"]["dark_two"])

            self.plot_design_grid.addWidget(plotter, 0, 0)

            msg = "Model exported."
            self.ui.update_logger(msg)
        else:
            msg = f"Failed backend call: {self.main_window.url}"
            self.ui.update_logger(msg)



from PySide6.QtCore import QThread
from PySide6.QtCore import Signal
from PySide6.QtWidgets import QComboBox
from PySide6.QtWidgets import QLabel
from PySide6.QtWidgets import QLineEdit
from PySide6.QtWidgets import QPushButton
from PySide6.QtWidgets import QWidget

# toolkit PySide6 Widgets
from ansys.aedt.toolkits.common.ui.utils.widgets import PyLabel
from ansys.aedt.core.generic.general_methods import _to_boolean
from windows.segmentation.segmentation_column import Ui_LeftColumn
from windows.segmentation.segmentation_page import Ui_Segmentation


class SegmentationThread(QThread):
    finished_signal = Signal(bool)

    def __init__(self, app, selected_project, selected_design):
        super().__init__()
        self.segmentation_menu = app
        self.main_window = app.main_window
        self.selected_project = selected_project
        self.selected_design = selected_design
        self.is_finished = False

    def run(self):
        self.is_finished = self.main_window.apply_segmentation(self.selected_project, self.selected_design)
        self.finished_signal.emit(self.is_finished)


class SkewThread(QThread):
    finished_signal = Signal(bool)

    def __init__(self, app, selected_project, selected_design):
        super().__init__()
        self.segmentation_menu = app
        self.main_window = app.main_window
        self.selected_project = selected_project
        self.selected_design = selected_design
        self.is_finished = False

    def run(self):
        self.is_finished = self.main_window.apply_skew(self.selected_project, self.selected_design)
        self.finished_signal.emit(self.is_finished)


class SegmentationMenu(object):
    def __init__(self, main_window):
        # General properties
        self.main_window = main_window
        self.ui = main_window.ui

        # Add page
        segmentation_menu_index = self.ui.add_page(Ui_Segmentation)
        self.ui.load_pages.pages.setCurrentIndex(segmentation_menu_index)
        self.segmentation_menu_widget = self.ui.load_pages.pages.currentWidget()

        # Add left column
        new_column_widget = QWidget()
        new_ui = Ui_LeftColumn()
        new_ui.setupUi(new_column_widget)
        self.ui.left_column.menus.menus.addWidget(new_column_widget)
        self.segmentation_column_widget = new_column_widget
        self.segmentation_column_vertical_layout = new_ui.segmentation_vertical_layout

        # Specific properties
        # Combo boxes
        self.apply_mesh_sheets = self.segmentation_menu_widget.findChild(QComboBox, "apply_mesh_sheets")
        self.is_skewed = self.segmentation_menu_widget.findChild(QComboBox, "is_skewed")
        self.magnets_material = self.segmentation_menu_widget.findChild(QComboBox, "magnets_material")
        self.rotor_material = self.segmentation_menu_widget.findChild(QComboBox, "rotor_material")
        self.stator_material = self.segmentation_menu_widget.findChild(QComboBox, "stator_material")
        # LineEdits
        self.rotor_slices = self.segmentation_menu_widget.findChild(QLineEdit, "rotor_slices")
        self.magnet_segments_per_slice = self.segmentation_menu_widget.findChild(QLineEdit, "magnet_segments_per_slice")
        self.mesh_sheets_number = self.segmentation_menu_widget.findChild(QLineEdit, "mesh_sheets_number")
        self.skew_angle = self.segmentation_menu_widget.findChild(QLineEdit, "skew_angle")
        # Labels
        self.projects_aedt_combo_label = self.segmentation_menu_widget.findChild(QLabel, "projects_aedt_combo_label")
        self.design_aedt_combo_label = self.segmentation_menu_widget.findChild(QLabel, "design_aedt_combo_label")
        self.apply_mesh_sheets_label = self.segmentation_menu_widget.findChild(QLabel, "apply_mesh_sheets_label")
        self.is_skewed_label = self.segmentation_menu_widget.findChild(QLabel, "is_skewed_label")
        self.magnets_material_label = self.segmentation_menu_widget.findChild(QLabel, "magnets_material_label")
        self.rotor_material_label = self.segmentation_menu_widget.findChild(QLabel, "rotor_material_label")
        self.stator_material_label = self.segmentation_menu_widget.findChild(QLabel, "stator_material_label")
        self.rotor_slices_label = self.segmentation_menu_widget.findChild(QLabel, "rotor_slices_label")
        self.magnet_segments_per_slice_label = self.segmentation_menu_widget.findChild(
            QLabel, "magnet_segments_per_slice_label"
        )
        self.mesh_sheets_number_label = self.segmentation_menu_widget.findChild(QLabel, "mesh_sheets_number_label")
        self.skew_angle_label = self.segmentation_menu_widget.findChild(QLabel, "skew_angle_label")
        # Buttons
        self.perform_segmentation_button = self.segmentation_menu_widget.findChild(QPushButton, "perform_segmentation")
        self.apply_skew_button = self.segmentation_menu_widget.findChild(QPushButton, "skew")
        self.segmentation_thread = None
        self.skew_thread = None

    def setup(self):
        # Modify theme
        app_color = self.main_window.ui.themes["app_color"]
        text_foreground = app_color["text_foreground"]
        combo_color = app_color["dark_three"]

        self.perform_segmentation_button.clicked.connect(self.segmentation_button_clicked)
        self.apply_skew_button.clicked.connect(self.skew_button_clicked)

        # UI from Designer
        # Combo boxes
        combo_box_style = """
            QComboBox {{
                border: none;
                padding: 10px;
                color: {_color};
                background-color: {_bg_color};
                selection-background-color: red;
                font-size: {_font_size}pt;
            }}
            QComboBox QAbstractItemView {{
                border: none;
                background-color: {_bg_color};
                color: {_color};
            }}
        """
        custom_style = combo_box_style.format(
            _color=text_foreground, _bg_color=combo_color, _font_size=self.main_window.properties.font["title_size"]
        )
        self.apply_mesh_sheets.setStyleSheet(custom_style)
        self.is_skewed.setStyleSheet(custom_style)
        self.magnets_material.setStyleSheet(custom_style)
        self.rotor_material.setStyleSheet(custom_style)
        self.stator_material.setStyleSheet(custom_style)

        # Multiplier line
        line_style = """
            QLineEdit {{
            border: none;
            padding: 10px;
            color: {_color};
            background-color: {_bg_color};
            selection-background-color: red;
            font-size: {_font_size}pt;
            }}
        """
        custom_style = line_style.format(
            _color=text_foreground, _bg_color=combo_color, _font_size=self.main_window.properties.font["title_size"]
        )
        self.rotor_slices.setStyleSheet(custom_style)
        self.magnet_segments_per_slice.setStyleSheet(custom_style)
        self.mesh_sheets_number.setStyleSheet(custom_style)
        self.skew_angle.setStyleSheet(custom_style)

        # Multiplier label button
        multiplier_label_style = """
                    QLabel {{
                    color: {_color};
                    font-size: {_font_size}pt;
                    font-weight: bold;
                    }}
                    """
        custom_style = multiplier_label_style.format(
            _color=text_foreground, _bg_color=combo_color, _font_size=self.main_window.properties.font["title_size"]
        )
        self.apply_mesh_sheets_label.setStyleSheet(custom_style)
        self.is_skewed_label.setStyleSheet(custom_style)
        self.magnets_material_label.setStyleSheet(custom_style)
        self.rotor_material_label.setStyleSheet(custom_style)
        self.stator_material_label.setStyleSheet(custom_style)
        self.rotor_slices_label.setStyleSheet(custom_style)
        self.magnet_segments_per_slice_label.setStyleSheet(custom_style)
        self.mesh_sheets_number_label.setStyleSheet(custom_style)
        self.skew_angle_label.setStyleSheet(custom_style)

        # Geometry label button
        select_geometry_label_style = """
                            QLabel {{
                            color: {_color};
                            font-size: {_font_size}pt;
                            font-weight: bold;
                            }}
                            """
        custom_style = select_geometry_label_style.format(
            _color=text_foreground, _bg_color=combo_color, _font_size=self.main_window.properties.font["title_size"]
        )
        self.perform_segmentation_button.setStyleSheet(custom_style)
        self.apply_skew_button.setStyleSheet(custom_style)

        # Set Column
        msg = "Segmentation"
        label_widget = PyLabel(
            text=msg,
            font_size=self.ui.app.properties.font["title_size"],
            color=self.ui.themes["app_color"]["text_description"],
        )
        self.segmentation_column_vertical_layout.addWidget(label_widget)

    def segmentation_button_clicked(self):
        if not self.main_window.check_connection():
            msg = "Backend not running."
            self.ui.update_logger(msg)
            return False

        if (
            self.segmentation_thread and self.segmentation_thread.isRunning()
            or self.skew_thread and self.skew_thread.isRunning()
            or self.main_window.backend_busy()
        ):
            msg = "Toolkit running"
            self.ui.update_logger(msg)
            self.main_window.logger.debug(msg)
            return False

        # if self.is_skewed.currentText() == "True":
        #     self.rotor_material.setEnabled(False)
        #     self.stator_material.setEnabled(False)
        #     self.rotor_slices.setEnabled(False)
        #     self.skew_angle.setEnabled(False)
        # else:
        #     self.rotor_material.setEnabled(True)
        #     self.stator_material.setEnabled(True)
        #     self.rotor_slices.setEnabled(True)
        #     self.skew_angle.setEnabled(True)

        be_properties = self.main_window.get_properties()

        be_properties["active_project"] = self.main_window.home_menu.project_combobox.currentText()
        be_properties["active_design"] = self.main_window.home_menu.design_combobox.currentText()

        be_properties["is_skewed"] = _to_boolean(self.is_skewed.currentText())
        if not be_properties["is_skewed"]:
            be_properties["rotor_material"] = self.rotor_material.currentText()
            be_properties["stator_material"] = self.stator_material.currentText()
            be_properties["rotor_slices"] = int(self.rotor_slices.text())
            be_properties["skew_angle"] = self.skew_angle.text()
        be_properties["apply_mesh_sheets"] = _to_boolean(self.apply_mesh_sheets.currentText())
        if be_properties["apply_mesh_sheets"]:
            be_properties["mesh_sheets_number"] = int(self.mesh_sheets_number.text())
        be_properties["magnets_material"] = self.magnets_material.currentText()
        be_properties["magnet_segments_per_slice"] = int(self.magnet_segments_per_slice.text())
        # be_properties.setup_to_analyze = self.setup_to_analyze.text()

        self.main_window.set_properties(be_properties)

        if be_properties.get("active_project"):
            self.ui.update_progress(0)
            selected_project = self.main_window.home_menu.project_combobox.currentText()
            selected_design = self.main_window.home_menu.design_combobox.currentText()

            # Start a separate thread for the backend call
            self.segmentation_thread = SegmentationThread(
                app=self,
                selected_project=selected_project,
                selected_design=selected_design,
            )
            self.segmentation_thread.finished_signal.connect(self.segmentation_process_finished)

            self.segmentation_thread.start()
        else:
            self.ui.update_logger("Toolkit not connected to AEDT.")

    def skew_button_clicked(self):
        if not self.main_window.check_connection():
            msg = "Backend not running."
            self.ui.update_logger(msg)
            return False

        if (self.skew_thread and self.skew_thread.isRunning()
            or self.segmentation_thread and self.segmentation_thread.isRunning()
            or self.main_window.backend_busy()):
            msg = "Toolkit running"
            self.ui.update_logger(msg)
            self.main_window.logger.debug(msg)
            return False

        be_properties = self.main_window.get_properties()

        if be_properties.get("active_project"):
            self.ui.update_progress(0)
            selected_project = self.main_window.home_menu.project_combobox.currentText()
            selected_design = self.main_window.home_menu.design_combobox.currentText()

            self.skew_thread = SkewThread(
                app=self,
                selected_project=selected_project,
                selected_design=selected_design,
            )
            self.skew_thread.finished_signal.connect(self.skew_process_finished)

            self.skew_thread.start()
        else:
            self.ui.update_logger("Toolkit not connected to AEDT.")

    def segmentation_process_finished(self):
        self.ui.update_progress(100)
        properties = self.main_window.get_properties()
        active_design = properties["active_design"]

        self.main_window.home_menu.update_design()
        self.main_window.home_menu.design_combobox.setCurrentText(active_design)

        if self.segmentation_thread.is_finished:
            msg = "Segmentation successful."
            self.ui.update_logger(msg)
        else:
            msg = f"Failed backend call: {self.main_window.url}"
            self.ui.update_logger(msg)

    def skew_process_finished(self):
        self.ui.update_progress(100)
        selected_project = self.main_window.home_menu.project_combobox.currentText()
        selected_design = self.main_window.home_menu.design_combobox.currentText()

        properties = self.main_window.get_properties()
        active_project = self.main_window.get_project_name(properties["active_project"])
        active_design = properties["active_design"]
        if active_project != selected_project or active_design != selected_design:
            self.main_window.home_menu.update_project()
            self.main_window.home_menu.update_design()

        if self.skew_thread.is_finished:
            msg = "Skew successful."
            self.ui.update_logger(msg)
        else:
            msg = f"Failed backend call: {self.main_window.url}"
            self.ui.update_logger(msg)

    def get_materials(self):
        backend_properties = self.ui.app.get_properties()
        materials = []
        if backend_properties.get("active_project") and backend_properties.get("active_design"):
            materials = self.main_window.get_materials()
        if materials:
            for mat in materials:
                self.magnets_material.addItem(mat)
                self.rotor_material.addItem(mat)
                self.stator_material.addItem(mat)



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'segmentation_page.ui'
##
## Created by: Qt User Interface Compiler version 6.6.3
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QComboBox, QHBoxLayout, QLabel,
    QLineEdit, QPushButton, QSizePolicy, QSpacerItem,
    QVBoxLayout, QWidget)

class Ui_Segmentation(object):
    def setupUi(self, Segmentation):
        if not Segmentation.objectName():
            Segmentation.setObjectName(u"Segmentation")
        Segmentation.resize(627, 456)
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Segmentation.sizePolicy().hasHeightForWidth())
        Segmentation.setSizePolicy(sizePolicy)
        Segmentation.setProperty("segmentation", QRect(0, 0, 890, 1178))
        self.verticalLayout = QVBoxLayout(Segmentation)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.horizontalLayout_ = QHBoxLayout()
        self.horizontalLayout_.setObjectName(u"horizontalLayout_")
        self.verticalLayout_2 = QVBoxLayout()
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.horizontalLayout_32 = QHBoxLayout()
        self.horizontalLayout_32.setObjectName(u"horizontalLayout_32")
        self.apply_mesh_sheets_label = QLabel(Segmentation)
        self.apply_mesh_sheets_label.setObjectName(u"apply_mesh_sheets_label")

        self.horizontalLayout_32.addWidget(self.apply_mesh_sheets_label)

        self.horizontalSpacer_2 = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_32.addItem(self.horizontalSpacer_2)

        self.apply_mesh_sheets = QComboBox(Segmentation)
        self.apply_mesh_sheets.addItem("")
        self.apply_mesh_sheets.addItem("")
        self.apply_mesh_sheets.setObjectName(u"apply_mesh_sheets")
        sizePolicy1 = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Expanding)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.apply_mesh_sheets.sizePolicy().hasHeightForWidth())
        self.apply_mesh_sheets.setSizePolicy(sizePolicy1)
        self.apply_mesh_sheets.setMinimumSize(QSize(300, 0))

        self.horizontalLayout_32.addWidget(self.apply_mesh_sheets)


        self.verticalLayout_2.addLayout(self.horizontalLayout_32)

        self.horizontalLayout_4 = QHBoxLayout()
        self.horizontalLayout_4.setObjectName(u"horizontalLayout_4")
        self.is_skewed_label = QLabel(Segmentation)
        self.is_skewed_label.setObjectName(u"is_skewed_label")

        self.horizontalLayout_4.addWidget(self.is_skewed_label)

        self.horizontalSpacer_5 = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_4.addItem(self.horizontalSpacer_5)

        self.is_skewed = QComboBox(Segmentation)
        self.is_skewed.addItem("")
        self.is_skewed.addItem("")
        self.is_skewed.setObjectName(u"is_skewed")
        sizePolicy1.setHeightForWidth(self.is_skewed.sizePolicy().hasHeightForWidth())
        self.is_skewed.setSizePolicy(sizePolicy1)
        self.is_skewed.setMinimumSize(QSize(300, 0))

        self.horizontalLayout_4.addWidget(self.is_skewed)


        self.verticalLayout_2.addLayout(self.horizontalLayout_4)

        self.horizontalLayout_7 = QHBoxLayout()
        self.horizontalLayout_7.setObjectName(u"horizontalLayout_7")
        self.magnets_material_label = QLabel(Segmentation)
        self.magnets_material_label.setObjectName(u"magnets_material_label")

        self.horizontalLayout_7.addWidget(self.magnets_material_label)

        self.horizontalSpacer_6 = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_7.addItem(self.horizontalSpacer_6)

        self.magnets_material = QComboBox(Segmentation)
        self.magnets_material.addItem("")
        self.magnets_material.setObjectName(u"magnets_material")
        sizePolicy1.setHeightForWidth(self.magnets_material.sizePolicy().hasHeightForWidth())
        self.magnets_material.setSizePolicy(sizePolicy1)
        self.magnets_material.setMinimumSize(QSize(300, 0))

        self.horizontalLayout_7.addWidget(self.magnets_material)


        self.verticalLayout_2.addLayout(self.horizontalLayout_7)

        self.horizontalLayout_20 = QHBoxLayout()
        self.horizontalLayout_20.setObjectName(u"horizontalLayout_20")
        self.rotor_material_label = QLabel(Segmentation)
        self.rotor_material_label.setObjectName(u"rotor_material_label")

        self.horizontalLayout_20.addWidget(self.rotor_material_label)

        self.horizontalSpacer_7 = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_20.addItem(self.horizontalSpacer_7)

        self.rotor_material = QComboBox(Segmentation)
        self.rotor_material.addItem("")
        self.rotor_material.setObjectName(u"rotor_material")
        sizePolicy1.setHeightForWidth(self.rotor_material.sizePolicy().hasHeightForWidth())
        self.rotor_material.setSizePolicy(sizePolicy1)
        self.rotor_material.setMinimumSize(QSize(300, 0))

        self.horizontalLayout_20.addWidget(self.rotor_material)


        self.verticalLayout_2.addLayout(self.horizontalLayout_20)

        self.horizontalLayout_20a = QHBoxLayout()
        self.horizontalLayout_20a.setObjectName(u"horizontalLayout_20a")
        self.stator_material_label = QLabel(Segmentation)
        self.stator_material_label.setObjectName(u"stator_material_label")

        self.horizontalLayout_20a.addWidget(self.stator_material_label)

        self.horizontalSpacer_8 = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_20a.addItem(self.horizontalSpacer_8)

        self.stator_material = QComboBox(Segmentation)
        self.stator_material.addItem("")
        self.stator_material.setObjectName(u"stator_material")
        sizePolicy1.setHeightForWidth(self.stator_material.sizePolicy().hasHeightForWidth())
        self.stator_material.setSizePolicy(sizePolicy1)
        self.stator_material.setMinimumSize(QSize(300, 0))

        self.horizontalLayout_20a.addWidget(self.stator_material)


        self.verticalLayout_2.addLayout(self.horizontalLayout_20a)

        self.horizontalLayout_21 = QHBoxLayout()
        self.horizontalLayout_21.setObjectName(u"horizontalLayout_21")
        self.rotor_slices_label = QLabel(Segmentation)
        self.rotor_slices_label.setObjectName(u"rotor_slices_label")

        self.horizontalLayout_21.addWidget(self.rotor_slices_label)

        self.horizontalSpacer_21 = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_21.addItem(self.horizontalSpacer_21)

        self.rotor_slices = QLineEdit(Segmentation)
        self.rotor_slices.setObjectName(u"rotor_slices")
        sizePolicy1.setHeightForWidth(self.rotor_slices.sizePolicy().hasHeightForWidth())
        self.rotor_slices.setSizePolicy(sizePolicy1)
        self.rotor_slices.setMinimumSize(QSize(300, 0))

        self.horizontalLayout_21.addWidget(self.rotor_slices)


        self.verticalLayout_2.addLayout(self.horizontalLayout_21)

        self.horizontalLayout_22 = QHBoxLayout()
        self.horizontalLayout_22.setObjectName(u"horizontalLayout_22")
        self.magnet_segments_per_slice_label = QLabel(Segmentation)
        self.magnet_segments_per_slice_label.setObjectName(u"magnet_segments_per_slice_label")

        self.horizontalLayout_22.addWidget(self.magnet_segments_per_slice_label)

        self.horizontalSpacer_22 = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_22.addItem(self.horizontalSpacer_22)

        self.magnet_segments_per_slice = QLineEdit(Segmentation)
        self.magnet_segments_per_slice.setObjectName(u"magnet_segments_per_slice")
        sizePolicy1.setHeightForWidth(self.magnet_segments_per_slice.sizePolicy().hasHeightForWidth())
        self.magnet_segments_per_slice.setSizePolicy(sizePolicy1)
        self.magnet_segments_per_slice.setMinimumSize(QSize(300, 0))

        self.horizontalLayout_22.addWidget(self.magnet_segments_per_slice)


        self.verticalLayout_2.addLayout(self.horizontalLayout_22)

        self.horizontalLayout_33 = QHBoxLayout()
        self.horizontalLayout_33.setObjectName(u"horizontalLayout_33")
        self.mesh_sheets_number_label = QLabel(Segmentation)
        self.mesh_sheets_number_label.setObjectName(u"mesh_sheets_number_label")

        self.horizontalLayout_33.addWidget(self.mesh_sheets_number_label)

        self.horizontalSpacer_3 = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_33.addItem(self.horizontalSpacer_3)

        self.mesh_sheets_number = QLineEdit(Segmentation)
        self.mesh_sheets_number.setObjectName(u"mesh_sheets_number")
        sizePolicy1.setHeightForWidth(self.mesh_sheets_number.sizePolicy().hasHeightForWidth())
        self.mesh_sheets_number.setSizePolicy(sizePolicy1)
        self.mesh_sheets_number.setMinimumSize(QSize(300, 0))

        self.horizontalLayout_33.addWidget(self.mesh_sheets_number)


        self.verticalLayout_2.addLayout(self.horizontalLayout_33)

        self.horizontalLayout_23 = QHBoxLayout()
        self.horizontalLayout_23.setObjectName(u"horizontalLayout_23")
        self.skew_angle_label = QLabel(Segmentation)
        self.skew_angle_label.setObjectName(u"skew_angle_label")

        self.horizontalLayout_23.addWidget(self.skew_angle_label)

        self.horizontalSpacer_23 = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_23.addItem(self.horizontalSpacer_23)

        self.skew_angle = QLineEdit(Segmentation)
        self.skew_angle.setObjectName(u"skew_angle")
        sizePolicy1.setHeightForWidth(self.skew_angle.sizePolicy().hasHeightForWidth())
        self.skew_angle.setSizePolicy(sizePolicy1)
        self.skew_angle.setMinimumSize(QSize(300, 0))

        self.horizontalLayout_23.addWidget(self.skew_angle)


        self.verticalLayout_2.addLayout(self.horizontalLayout_23)

        self.verticalSpacer = QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Maximum)

        self.verticalLayout_2.addItem(self.verticalSpacer)

        self.perform_segmentation = QPushButton(Segmentation)
        self.perform_segmentation.setObjectName(u"perform_segmentation")
        self.perform_segmentation.setEnabled(True)
        sizePolicy.setHeightForWidth(self.perform_segmentation.sizePolicy().hasHeightForWidth())
        self.perform_segmentation.setSizePolicy(sizePolicy)
        self.perform_segmentation.setMinimumSize(QSize(0, 40))
        self.perform_segmentation.setMaximumSize(QSize(16777215, 16777215))

        self.verticalLayout_2.addWidget(self.perform_segmentation)

        self.skew = QPushButton(Segmentation)
        self.skew.setObjectName(u"skew")
        self.skew.setEnabled(True)
        sizePolicy.setHeightForWidth(self.skew.sizePolicy().hasHeightForWidth())
        self.skew.setSizePolicy(sizePolicy)
        self.skew.setMinimumSize(QSize(0, 40))

        self.verticalLayout_2.addWidget(self.skew)


        self.horizontalLayout_.addLayout(self.verticalLayout_2)

        self.horizontalSpacer_4 = QSpacerItem(40, 20, QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_.addItem(self.horizontalSpacer_4)


        self.verticalLayout.addLayout(self.horizontalLayout_)


        self.retranslateUi(Segmentation)

        QMetaObject.connectSlotsByName(Segmentation)
    # setupUi

    def retranslateUi(self, Segmentation):
        Segmentation.setWindowTitle(QCoreApplication.translate("Segmentation", u"Form", None))
        self.apply_mesh_sheets_label.setText(QCoreApplication.translate("Segmentation", u"Apply Mesh Sheets", None))
        self.apply_mesh_sheets.setItemText(0, QCoreApplication.translate("Segmentation", u"False", None))
        self.apply_mesh_sheets.setItemText(1, QCoreApplication.translate("Segmentation", u"True", None))

        self.is_skewed_label.setText(QCoreApplication.translate("Segmentation", u"Is Skewed", None))
        self.is_skewed.setItemText(0, QCoreApplication.translate("Segmentation", u"False", None))
        self.is_skewed.setItemText(1, QCoreApplication.translate("Segmentation", u"True", None))

        self.magnets_material_label.setText(QCoreApplication.translate("Segmentation", u"Magnet Material", None))
        self.magnets_material.setItemText(0, QCoreApplication.translate("Segmentation", u"--Select Material--", None))

        self.rotor_material_label.setText(QCoreApplication.translate("Segmentation", u"Rotor Material", None))
        self.rotor_material.setItemText(0, QCoreApplication.translate("Segmentation", u"--Select Material--", None))

        self.stator_material_label.setText(QCoreApplication.translate("Segmentation", u"Stator Material", None))
        self.stator_material.setItemText(0, QCoreApplication.translate("Segmentation", u"--Select Material--", None))

        self.rotor_slices_label.setText(QCoreApplication.translate("Segmentation", u"Rotor Slices", None))
        self.magnet_segments_per_slice_label.setText(QCoreApplication.translate("Segmentation", u"Magnet Segments Per Slice", None))
        self.mesh_sheets_number_label.setText(QCoreApplication.translate("Segmentation", u"Mesh Sheets Number", None))
        self.skew_angle_label.setText(QCoreApplication.translate("Segmentation", u"Skew Angle (deg)", None))
        self.perform_segmentation.setText(QCoreApplication.translate("Segmentation", u"Perform Segmentation", None))
        self.skew.setText(QCoreApplication.translate("Segmentation", u"Apply Skew", None))
    # retranslateUi




# Copyright (C) 2023 - 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import atexit
import os
import sys

from ansys.aedt.toolkits.common.utils import check_backend_communication
from ansys.aedt.toolkits.common.utils import clean_python_processes
from ansys.aedt.toolkits.common.utils import find_free_port
from ansys.aedt.toolkits.common.utils import is_server_running
from ansys.aedt.toolkits.common.utils import process_desktop_properties
from ansys.aedt.toolkits.common.utils import server_actions
from ansys.aedt.toolkits.common.utils import wait_for_server

from ansys.aedt.toolkits.magnet_segmentation import backend
from ansys.aedt.toolkits.magnet_segmentation import ui
from ansys.aedt.toolkits.magnet_segmentation.backend.models import properties as backend_properties
from ansys.aedt.toolkits.magnet_segmentation.ui.models import properties as frontend_properties

# Set resolution and user interface theme
os.environ["AEDT_TOOLKIT_HIGH_RESOLUTION"] = "False"
os.environ["AEDT_TOOLKIT_THEME"] = "ansys_dark.json"

# Define global variables or constants
backend_file = os.path.join(backend.__path__[0], "run_backend.py")
frontend_file = os.path.join(ui.__path__[0], "run_frontend.py")

is_linux = os.name == "posix"
new_port = find_free_port(backend_properties.url, backend_properties.port)
if not new_port:
    raise Exception("No free ports in {}".format(backend_properties.url))

frontend_properties.backend_port = new_port
url = frontend_properties.backend_url
port = frontend_properties.backend_port
python_path = sys.executable
backend_command = [python_path, backend_file, url, str(port)]
frontend_command = [python_path, frontend_file, url, str(port)]
url_call = f"http://{url}:{port}"

# Main Execution

# Clean python process when script ends
atexit.register(clean_python_processes, url, port)

# Check if backend is already running
is_server_busy = is_server_running(server=url, port=port)
if is_server_busy:
    raise Exception("There is a process running in: {}".format(url_call))

# Launch backend thread
backend_thread = server_actions(backend_command, "template_backend", is_linux)

# Make a first call to the backend to check the communication
backend_communication_flag = check_backend_communication(url_call)
count = 0
while not backend_communication_flag and count < 10:
    backend_communication_flag = check_backend_communication(url_call)
    count += 1
if not backend_communication_flag:
    raise Exception("Backend communication is not working.")

# Connect to AEDT session if arguments or environment variables are passed
process_desktop_properties(is_linux, url_call)

# Launch frontend thread
frontend_thread = server_actions(frontend_command, "template_frontend", is_linux)

# Check if backend is running. Try every 1 second with a timeout of 10 seconds.
backend_flag = wait_for_server(server=url, port=port)
if not backend_flag:
    raise Exception("There is a process running in: {}".format(url_call))

# Keep frontend thread alive until it is closed
frontend_thread.join()



# Copyright (C) 2023 - 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from ansys.aedt.toolkits.common.ui.actions_generic import FrontendGeneric
from ansys.aedt.toolkits.common.ui.logger_handler import logger

# import os.path
import requests


class Frontend(FrontendGeneric):
    def __init__(self):
        FrontendGeneric.__init__(self)

    def check_segmentation_compatibility(self):
        """Check compatibility with segmentation call.

        An AEDT file is compatible with the segmentation call if it has defined
        design settings 'SymmetryFactor' and 'HalfAxial'.
        """
        SYMMETRY_FACTOR_BEGIN = "VariableProp('SymmetryFactor'"
        HALF_AXIAL_BEGIN = "VariableProp('HalfAxial'"
        found_data = {target: False for target in [SYMMETRY_FACTOR_BEGIN, HALF_AXIAL_BEGIN]}

        be_properties = self.get_properties()
        if not be_properties.active_project:
            return
        else:
            with open(be_properties.active_project, "r") as file:
                lines = file.readlines()
            lines = [line.strip() for line in lines]
        be_properties = self.get_properties()
        with open(be_properties.active_project, "r") as file:
            lines = file.readlines()
        lines = [line.strip() for line in lines]

        for line in lines:
            for target in [SYMMETRY_FACTOR_BEGIN, HALF_AXIAL_BEGIN]:
                if line.lstrip().startswith(target):
                    found_data[target] = True

        res = all(found_data.values())
        if res:
            logger.debug("Selected AEDT file is compatible with segmentation call")
        else:
            logger.debug("Selected AEDT file is not compatible with segmentation call")
        return res

    def check_skew_compatibility(self):
        """Check compatibility with skew call.

        An AEDT file is compatible with the skew call if the name of the shaft
        is 'Shaft'.
        """
        SHAFT_LINES = ["$begin 'GeometryPart'", "$begin 'Attributes'", "Name='Shaft'"]

        be_properties = self.get_properties()
        with open(be_properties.active_project, "r") as file:
            lines = file.readlines()
        lines = [line.strip() for line in lines]

        for i in range(len(lines) - len(SHAFT_LINES) + 1):
            if lines[i : i + len(SHAFT_LINES)] == SHAFT_LINES:
                logger.debug("Selected AEDT file is compatible with skew call")
                return True

        logger.debug("Selected AEDT file is not compatible with skew call")
        return False

    def browse_and_check_for_aedt_project(self):
        be_properties = self.get_properties()
        super().browse_for_aedt_project()
        segmentation_compatibility = self.check_segmentation_compatibility()
        if not segmentation_compatibility and segmentation_compatibility is not None:
            self.write_log_line(
                f"[Warning] AEDT file '{be_properties.active_project}' is not compatible with segmentation."
            )
            self.write_log_line("Please, ensure that 'SymmetryFactor' and 'HalfAxial' are defined")
        elif segmentation_compatibility:
            skew_compatibility = self.check_skew_compatibility()
            if not skew_compatibility:
                self.write_log_line(
                    f"[Warning] AEDT file '{be_properties.active_project}' is not compatible with skew."
                )
                self.write_log_line("Please, ensure that the name of the shaft is 'Shaft'")

    def apply_segmentation(self, project_selected=None, design_selected=None):
        be_properties = self.get_properties()
        if project_selected and design_selected:
            for project in be_properties["project_list"]:
                if self.get_project_name(project) == project_selected:
                    be_properties["active_project"] = project
                    if project_selected in list(be_properties["design_list"].keys()):
                        designs = be_properties["design_list"][project_selected]
                        for design in designs:
                            if design_selected == design:
                                be_properties["active_design"] = design
                                break
                    break
        else:
            msg = "Please load a valid AEDT project."
            logger.info(msg)

        self.set_properties(be_properties)
        try:
            segmentation_response = requests.post(self.url + "/apply_segmentation")
            if segmentation_response.ok:
                # self.find_design_names()
                # self.skew.setEnabled(True)
                msg = "Apply segmentation call successful"
                logger.info(msg)
                return True
            else:
                msg = f"Apply segmentation call failed"
                logger.error(msg)
                return False
        except requests.exceptions.RequestException:
            logger.error("Apply segmentation call failed")

    def apply_skew(self, project_selected=None, design_selected=None):
        be_properties = self.get_properties()
        if project_selected and design_selected:
            for project in be_properties["project_list"]:
                if self.get_project_name(project) == project_selected:
                    be_properties["active_project"] = project
                    if project_selected in list(be_properties["design_list"].keys()):
                        designs = be_properties["design_list"][project_selected]
                        for design in designs:
                            if design_selected == design:
                                be_properties["active_design"] = design
                                break
                    break
        else:
            msg = "Please load a valid AEDT project."
            logger.info(msg)

        self.set_properties(be_properties)

        if be_properties["is_skewed"]:
            msg = "Model is already skewed."
            logger.info(msg)
            return

        if be_properties["skew_angle"]:
            try:
                response = requests.post(self.url + "/apply_skew")
                if response.ok:
                    msg = "Apply skew call successful."
                    logger.info(msg)
                    return True
                else:
                    msg = "Apply skew call failed."
                    logger.error(msg)
                    return False
            except requests.exceptions.RequestException:
                logger.error("Apply skew call failed.")

    def val_check_and_analysis(self, project_selected=None, design_selected=None):
        be_properties = self.get_properties()
        if project_selected and design_selected:
            for project in be_properties["project_list"]:
                if self.get_project_name(project) == project_selected:
                    be_properties["active_project"] = project
                    if project_selected in list(be_properties["design_list"].keys()):
                        designs = be_properties["design_list"][project_selected]
                        for design in designs:
                            if design_selected == design:
                                be_properties["active_design"] = design
                                break
                    break
        else:
            msg = "Please load a valid AEDT project."
            logger.info(msg)

        self.set_properties(be_properties)

        try:
            response = requests.post(self.url + "/validate_analyze")
            if response.ok:
                msg = "Validate and analyze call successful"
                logger.info(msg)
                return True
            else:
                msg = "Validate and analyze call failed"
                logger.error(msg)
                return False
        except requests.exceptions.RequestException:
            logger.error("Validate and analyze call failed")

    def get_report(self, project_selected=None, design_selected=None):
        be_properties = self.get_properties()
        if project_selected and design_selected:
            for project in be_properties["project_list"]:
                if self.get_project_name(project) == project_selected:
                    be_properties["active_project"] = project
                    if project_selected in list(be_properties["design_list"].keys()):
                        designs = be_properties["design_list"][project_selected]
                        for design in designs:
                            if design_selected == design:
                                be_properties["active_design"] = design
                                break
                    break
        else:
            msg = "Please load a valid AEDT project."
            logger.info(msg)

        self.set_properties(be_properties)

        try:
            response = requests.get(self.url + "/magnet_loss")
            if response.ok:
                msg = "Magnet loss call successful."
                logger.info(msg)
                return True
            else:
                msg = "Magnet loss call failed."
                logger.error(msg)
                return False
        except requests.exceptions.RequestException:
            logger.error("Magnet loss call failed.")

    def get_materials(self):
        try:
            thread_response = self.wait_thread(60)
            if thread_response:
                response = requests.get(self.url + "/project_materials")
                if response.ok:
                    msg = "Load materials call successful."
                    logger.info(msg)
                    return response.json()
                else:
                    msg = "Load materials call failed."
                    logger.error(msg)
                    return False
            # else toolkit busy

        except requests.exceptions.RequestException:
            logger.error("Load materials call failed.")

    def get_design_setups(self):
        try:
            response = requests.get(self.url + "/design_setups")
            if response.ok:
                msg = "Get design setups call successful."
                logger.info(msg)
                return response.json()
            else:
                msg = "Get design setups call failed."
                logger.error(msg)
                return False
        except requests.exceptions.RequestException:
            logger.error("Get design setups call failed.")



# Copyright (C) 2023 - 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from operator import attrgetter

from ansys.aedt.core.application.variables import decompose_variable_value
from ansys.aedt.core.generic.numbers import Quantity
from ansys.aedt.core.modeler.cad.modeler import CoordinateSystem
from ansys.aedt.core.modeler.cad.modeler import FaceCoordinateSystem
from ansys.aedt.core.modeler.geometry_operators import GeometryOperators as go
from ansys.aedt.toolkits.common.backend.api import AEDTCommon

from ansys.aedt.toolkits.magnet_segmentation.backend.models import properties


class AEDTWorkflow(AEDTCommon):
    """Controls the AEDT toolkit workflow.

    This class provides methods for connecting to a selected design,
    segment and skew the motor.

    Examples
    --------
        >>> from ansys.aedt.toolkits.magnet_segmentation.backend.api import AEDTWorkflow
        >>> toolkit = AEDTWorkflow()
        >>> msg1 = toolkit.launch_aedt()
        >>> toolkit.wait_to_be_idle()
        >>> toolkit.segmentation()
        >>> toolkit.apply_skew()
        >>> toolkit.release_aedt(True, True)
    """

    def __init__(self):
        AEDTCommon.__init__(self, properties)
        self.properties = properties

    # @thread.launch_thread
    def segmentation(self):
        """Apply object segmentation.

        This method automatically segments the rotor, rotor pockets, and magnets.

        .. warning::
            This method only works if the AEDT active project has
            ``SymmetryFactor`` and ``HalfAxial`` design settings defined.

        Returns
        -------
        bool
            ``True`` when successful, ``False`` when failed.
        """
        self.connect_design()

        # Requirements: Design needs  a design variable "HalfAxial"
        if self.aedtapp["HalfAxial"] == "1":
            self.aedtapp["HalfAxial"] = "0"

        for obj in self.aedtapp.modeler.unclassified_objects:
            obj.model = False

        self.aedtapp.set_active_design(self.properties.active_design)
        self.aedtapp.duplicate_design(self.properties.active_design)
        self.properties.active_design = self.aedtapp.design_name
        self.aedtapp.set_active_design(self.properties.active_design)
        self.properties.design_list[self.aedtapp.project_name].append(self.properties.active_design)
        self.set_properties(self.properties.model_dump())

        # try:
        if [bound for bound in self.aedtapp.boundaries if bound.type == "Insulating"]:
            for bound in self.aedtapp.boundaries[:]:
                if (
                    bound.type == "Insulating"
                    and bound.name in self.aedtapp.odesign.GetChildObject("Boundaries").GetChildNames()
                ):
                    bound.delete()

        vacuum_objects = [x for x in self.aedtapp.modeler.get_objects_by_material("vacuum") if x.object_type == "Solid"]
        rotor_pockets = self._get_rotor_pockets(vacuum_objects)

        # If model is already skewed only magnets can be segmented
        if not self.properties.is_skewed:
            magnets = self.aedtapp.modeler.get_objects_by_material(self.properties.magnets_material)
            if self.properties.rotor_material == self.properties.stator_material:
                rotor_stator = self.aedtapp.modeler.get_objects_by_material(self.properties.rotor_material)
                stator_obj = max(rotor_stator, key=attrgetter("volume"))
                rotor = [
                    x
                    for x in self.aedtapp.modeler.get_objects_by_material(self.properties.rotor_material)
                    if x != stator_obj and x.name != "Shaft"
                ][0]
            else:
                rotor = self.aedtapp.modeler.get_objects_by_material(self.properties.rotor_material)[0]

            if self.properties.rotor_slices > 1:
                # rotor segmentation
                rotor_slices = self.aedtapp.modeler.objects_segmentation(
                    rotor.id, segments=self.properties.rotor_slices, apply_mesh_sheets=False
                )
                # rotor and rotor pockets split
                rotor_objs = [rotor.name]
                magnets_names = [x.name for x in magnets]
                if len(rotor_pockets) > 0:
                    rotor_pockets_names = [x.name for x in rotor_pockets]
                for slice in rotor_slices[rotor.name]:
                    rotor_objs = self.aedtapp.modeler.split(rotor_objs, sides="Both", tool=slice.faces[0])
                    magnets_names = self.aedtapp.modeler.split(magnets_names, sides="Both", tool=slice.faces[0])
                    if len(rotor_pockets) > 0:
                        rotor_pockets_names = self.aedtapp.modeler.split(
                            rotor_pockets_names, sides="Both", tool=slice.faces[0]
                        )
                    self.aedtapp.modeler.delete_objects_containing(slice.name)
                rotor_slices.clear()

        magnets = self.aedtapp.modeler.get_objects_by_material(self.properties.magnets_material)
        for magnet in magnets:
            faces = []
            mesh_sheets = []
            cs = self.aedtapp.modeler.duplicate_coordinate_system_to_global(magnet.part_coordinate_system)
            magnet.part_coordinate_system = cs.name
            self.aedtapp.modeler.set_working_coordinate_system("Global")
            objects_segmentation = self.aedtapp.modeler.objects_segmentation(
                magnet.id,
                segments=self.properties.magnet_segments_per_slice,
                apply_mesh_sheets=self.properties.apply_mesh_sheets,
                mesh_sheets=self.properties.mesh_sheets_number,
            )
            if self.properties.apply_mesh_sheets:
                magnet_segments = objects_segmentation[0]
                mesh_sheets.extend([s.name for s in objects_segmentation[1][magnet.name]])
            else:
                magnet_segments = objects_segmentation
            for face in magnet_segments[magnet.name]:
                obj = self.aedtapp.modeler.create_object_from_face(face.top_face_z)
                faces.append(obj.top_face_z)
            [face.delete() for face in magnet_segments[magnet.name]]
            self.aedtapp.assign_insulating(faces, "{}_segments".format(magnet.name))
            if isinstance(cs, CoordinateSystem):
                self._update_cs(cs)

        magnets = self.aedtapp.modeler.get_objects_by_material(self.properties.magnets_material)
        segments = self.aedtapp.modeler.get_objects_in_group("Insulating")

        self.properties.objects.extend([m.name for m in magnets])
        self.properties.objects.extend(segments)
        # self.properties.objects.extend(mesh_sheets)
        self.set_properties(self.properties.model_dump())

        self.aedtapp.save_project()
        self.release_aedt(False, False)
        return True

    # @thread.launch_thread
    def apply_skew(self):
        """Apply skew to rotor slices.

        .. warning::
            This method only works if the active AEDT project contains a shaft
            named ``Shaft``.

        Returns
        -------
        bool
            ``True`` when successful, ``False`` when failed.
        """
        try:
            self.connect_design()

            magnets = self.aedtapp.modeler.get_objects_by_material(self.properties.magnets_material)
            rotor_objects = self.aedtapp.modeler.get_objects_by_material(self.properties.rotor_material)
            # Independent and dependent boundary conditions can be assigned either to an object or an object face
            independent = [bound for bound in self.aedtapp.boundaries if bound.type == "Independent"]
            dependent = [bound for bound in self.aedtapp.boundaries if bound.type == "Dependent"]
            # If no indep. o dep. boundary are found it means that the symmetry factor is 1 -> whole motor.
            if independent or dependent:
                bound_indep_props = independent[0].props
                if "Objects" in bound_indep_props:
                    bound_indep_id = bound_indep_props["Objects"][0]
                    indep = self.aedtapp.modeler.objects[bound_indep_id]
                elif "Faces" in bound_indep_props:
                    bound_indep_id = bound_indep_props["Faces"][0]
                    obj = [o for o in self.aedtapp.modeler.object_list for f in o.faces if f.id == bound_indep_id][0]
                    indep = [f for f in obj.faces if f.id == bound_indep_id][0]
                bound_dep_props = dependent[0].props
                if "Objects" in bound_indep_props:
                    bound_dep_id = bound_dep_props["Objects"][0]
                    dep = self.aedtapp.modeler.objects[bound_dep_id]
                elif "Faces" in bound_indep_props:
                    bound_dep_id = bound_dep_props["Faces"][0]
                    obj = [o for o in self.aedtapp.modeler.object_list for f in o.faces if f.id == bound_dep_id][0]
                    dep = [f for f in obj.faces if f.id == bound_dep_id][0]

            # check whether stator has same rotor material
            if self.properties.rotor_material == self.properties.stator_material:
                stator_obj = max(rotor_objects, key=attrgetter("volume"))
                rotor_objects = [
                    x
                    for x in self.aedtapp.modeler.get_objects_by_material(self.properties.rotor_material)
                    if x != stator_obj and x.name != "Shaft"
                ]
            rotor_skew_ang = 0
            # rotate objects and apply skew
            for rotor_object in rotor_objects:
                if rotor_skew_ang != 0:
                    touching_objects = [self.aedtapp.modeler.objects_by_name[o] for o in rotor_object.touching_objects]
                    touching_magnets = list(set(touching_objects).intersection(magnets))
                    obj_in_bb = self.aedtapp.modeler.objects_in_bounding_box(
                        rotor_object.bounding_box, check_lines=False, check_sheets=False
                    )
                    if not touching_magnets:
                        touching_magnets = list(set(obj_in_bb).intersection(magnets))
                    for obj in touching_magnets:
                        magnet_cs = [
                            cs
                            for cs in self.aedtapp.modeler.coordinate_systems
                            if cs.name == obj.part_coordinate_system
                        ][0]
                        if isinstance(magnet_cs, CoordinateSystem):
                            magnet_cs.props["Phi"] = "{}+{}deg".format(magnet_cs.props["Phi"], rotor_skew_ang)
                        elif isinstance(magnet_cs, FaceCoordinateSystem):
                            magnet_cs.props["ZRotationAngle"] = "{}deg".format(rotor_skew_ang)
                        self.aedtapp.modeler.set_working_coordinate_system("Global")
                    for obj in obj_in_bb:
                        obj.rotate(axis="Z", angle=rotor_skew_ang)
                    rotor_object.rotate(axis="Z", angle=rotor_skew_ang)

                    # It means that indep. and dep. boundaries exist -> symmetry factor != 1
                    if independent and dependent:
                        split = self.aedtapp.modeler.split(assignment=rotor_object, sides="Both", tool=indep.id)
                        if [s for s in split if s not in self.aedtapp.modeler.objects_by_name]:
                            self.aedtapp.odesign.Undo()
                        split = self.aedtapp.modeler.split(assignment=rotor_object, sides="Both", tool=dep.id)
                        split_objects = [self.aedtapp.modeler.objects_by_name[obj] for obj in split]
                        # Get object with minimum volume
                        min_vol_object = min(split_objects, key=lambda x: x.volume).volume
                        # Get object whose volume is equal to min_vol_object
                        obj_rotate = [obj for obj in split_objects if round(obj.volume, 4) == round(min_vol_object, 4)][
                            0
                        ]
                        # duplicate around z axis (-360/symmetry_multiplier)
                        self.aedtapp.modeler.rotate(
                            obj_rotate, self.aedtapp.AXIS.Z, -360 / self.aedtapp.symmetry_multiplier
                        )
                        self.aedtapp.modeler.unite([split_objects[0], split_objects[1]])
                rotor_skew_ang += decompose_variable_value(self.properties.skew_angle)[0]

            self.release_aedt(False, False)
            return True
        except:
            return False

    def validate_and_analyze(self):
        """Validate and analyze the design.

        Returns
        -------
        bool
            ``True`` when successful, ``False`` when failed.
        """
        self.connect_design()

        if self.aedtapp.validate_simple():
            self.aedtapp.analyze_setup(self.properties.setup_to_analyze, use_auto_settings=False)
            self.release_aedt(False, False)
            return True
        else:
            return False

    def get_magnet_loss(self):
        """Get magnet loss.

        Returns
        -------
        dict
            dictionary containing the average magnet loss value when successful, ``False`` when failed.
        """
        self.connect_design()

        try:
            self.aedtapp.post.create_report(expressions="SolidLoss", plot_name="Losses", primary_sweep_variable="Time")
            data = self.aedtapp.post.get_solution_data(
                expressions="SolidLoss", primary_sweep_variable="Time", domain="Sweep"
            )
            avg = Quantity(
                expression=sum(data.data_magnitude()) / len(data.data_magnitude()), unit=data.units_data["SolidLoss"]
            )
            avg_w = avg.to("W")
            self.release_aedt(False, False)
            return str(avg_w)
        except:
            return False

    def _get_rotor_pockets(self, vacuum_objects):
        """Get the rotor pockets if any.

        Parameters
        ----------
        vacuum_objects : list
            List of vacuum objects.

        Returns
        -------
        list
            List of the class:`ansys.aedt.core.modeler.cad.object3d.Object3d` classes.
        """
        rotor_pockets = []
        for obj in vacuum_objects:
            obj_in_bb = self.aedtapp.modeler.objects_in_bounding_box(
                obj.bounding_box, check_lines=False, check_sheets=False
            )
            obj_in_bb.remove(obj)
            if isinstance(obj_in_bb, list) and len(obj_in_bb) == 1:
                rotor_pockets.append(obj)
        return rotor_pockets

    def _update_cs(self, cs):
        """Update the coordinate system to Euler ZYZ mode.

        Parameters
        ----------
        cs : :class:`ansys.aedt.core.modeler.Modeler.CoordinateSystem`
            Coordinate system.

        Returns
        -------
        bool
            ``True`` when successful, ``False`` when failed.
        """
        try:
            x_pointing = [
                decompose_variable_value(cs.props["XAxisXvec"])[0],
                decompose_variable_value(cs.props["XAxisYvec"])[0],
                decompose_variable_value(cs.props["XAxisZvec"])[0],
            ]
            y_pointing = [
                decompose_variable_value(cs.props["YAxisXvec"])[0],
                decompose_variable_value(cs.props["YAxisYvec"])[0],
                decompose_variable_value(cs.props["YAxisZvec"])[0],
            ]
            x, y, z = go.pointing_to_axis(x_pointing, y_pointing)
            phi, theta, psi = go.axis_to_euler_zyz(x, y, z)
            magnet_angle = go.rad2deg(phi)
            cs.change_cs_mode(2)
            cs.props["Phi"] = "{}deg".format(str(magnet_angle))
            cs.props["Psi"] = "90deg"
            cs.props["Theta"] = "0deg"
            cs.update()
            return True
        except:
            return False

    # @thread.launch_thread
    def _get_project_materials(self):
        """Get the project materials."""
        self.connect_design()

        if not self.aedtapp:
            return
        mats = self.aedtapp.materials.get_used_project_material_names()

        self.release_aedt(False, False)

        return mats

    def _get_design_setup_names(self):
        """Get the design setup names."""
        self.connect_design()

        setup_names = [setup.name for setup in self.aedtapp.setups]

        self.release_aedt(False, False)

        return setup_names



# Copyright (C) 2023 - 2024 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from ansys.aedt.toolkits.common.backend.multithreading_server import MultithreadingServer
from ansys.aedt.toolkits.common.backend.rest_api import app
from ansys.aedt.toolkits.common.backend.rest_api import jsonify
from ansys.aedt.toolkits.common.backend.rest_api import logger
from api import ToolkitBackend

toolkit_api = ToolkitBackend()


# ToolkitBackend entrypoints


@app.route("/project_materials", methods=["GET"])
def get_materials():
    logger.info("[GET] /Get project materials.")

    response = toolkit_api._get_project_materials()
    if response:
        return response
    else:
        return jsonify("Get project material was unsuccessful."), 500


@app.route("/design_setups", methods=["GET"])
def get_design_setups():
    logger.info("[GET] /Get design setups.")

    response = toolkit_api._get_design_setup_names()
    if response:
        return response
    else:
        return jsonify("Get design setups was unsuccessful."), 500


@app.route("/apply_segmentation", methods=["POST"])
def magnets_segmentation():
    logger.info("[POST] /Apply magnets segmentation.")

    response = toolkit_api.segmentation()
    if response:
        return (
            jsonify("Magnets segmentation applied successfully."),
            200,
        )
    else:
        return jsonify("Failure: Magnets segmentation application was unsuccessful."), 500


@app.route("/apply_skew", methods=["POST"])
def apply_skew():
    logger.info("[POST] /Apply skew.")

    response = toolkit_api.apply_skew()
    if response:
        return (
            jsonify("Skew angle applied successfully."),
            200,
        )
    else:
        return jsonify("Failure: Skew angle application was unsuccessful."), 500


@app.route("/validate_analyze", methods=["POST"])
def analyze_model():
    logger.info("[POST] /Validate and analyze AEDT model.")

    response = toolkit_api.validate_and_analyze()
    if response:
        return jsonify("AEDT model analyzed successfully."), 200
    else:
        return jsonify("Failure: AEDT model analysis was unsuccessful."), 500


@app.route("/magnet_loss", methods=["GET"])
def get_magnet_loss():
    logger.info("[GET] /Get magnet loss.")

    response = toolkit_api.get_magnet_loss()
    if response:
        return jsonify(f"Magnet Loss (avg)[W] = {response}"), 200
    else:
        return jsonify("Failure: Magnet loss calculation was unsuccessful."), 500


if __name__ == "__main__":
    app.debug = toolkit_api.properties.debug
    server = MultithreadingServer()
    server.run(host=toolkit_api.properties.url, port=toolkit_api.properties.port, app=app)
